import os
from os.path import join, exists

from syncloud.app import runner

from logger import get_logger


CERTIFICATES_PATH = '/root/.ssh'
KEYS_FILENAME = 'authorized_keys'
SYNCLOUD_KEY_FILENAME = 'id_rsa_syncloud_master'

SERVICE_NAME = 'ssh'
SERVICE_PROTOCOL = 'ssh'
SERVICE_TYPE = '_ssh._tcp'
SERVICE_PORT = 1022
SERVICE_URL = 'ssh'
BITS = 2048


class RemoteAccess:

    def __init__(self, insider, key_gen):
        self.key_gen = key_gen
        self.insider = insider
        self.logger = get_logger('RemoteAccess')

    def enable(self):

        self.copy_sshd()

        if not exists(CERTIFICATES_PATH):
            os.makedirs(CERTIFICATES_PATH)

        private, public = self.key_gen.generate('rsa', BITS)

        keys_filename = join(CERTIFICATES_PATH, KEYS_FILENAME)
        with open(keys_filename, 'a+') as keys_file:
            keys_file.write(public)
            keys_file.write('\n')

        self.insider.add_service(SERVICE_NAME, SERVICE_PROTOCOL, SERVICE_TYPE, SERVICE_PORT, SERVICE_URL)

        private_key_filename = join(CERTIFICATES_PATH, SYNCLOUD_KEY_FILENAME)

        with open(private_key_filename, 'w+') as private_key_file:
            private_key_file.write(private)

        return private

    def disable(self):
        self.insider.remove_service(SERVICE_NAME)

    def copy_sshd(self):
        self.key_gen.generate_into_file('rsa', '/etc/ssh/ssh_syncloud_host_rsa_key', BITS, overwrite=True)
        self.key_gen.generate_into_file('dsa', '/etc/ssh/ssh_syncloud_host_dsa_key', overwrite=True)
        self.key_gen.generate_into_file('ecdsa', '/etc/ssh/ssh_syncloud_host_ecdsa_key', overwrite=True)
        runner.call('remote-access-copy-sshd', self.logger, shell=True)